searchNodes=[{"doc":"A framework for defining Battlesnake-compatible rulesets and board positions.This top-level module is just a namespace. Check the submodules for all the interesting bits.","ref":"Snek.html","title":"Snek","type":"module"},{"doc":"A struct for representing a board position.This may be used to keep track of state in a game, each turn of the game producing the next board position.","ref":"Snek.Board.html","title":"Snek.Board","type":"module"},{"doc":"Returns a list of neighboring points adjascent to a point of origin.This excludes points that are outside of the board's boundaries.Examplesiex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.adjascent_neighbors(Board.Point.new(1, 1)) [ %Board.Point{x: 1, y: 0}, %Board.Point{x: 1, y: 2}, %Board.Point{x: 2, y: 1}, %Board.Point{x: 0, y: 1} ] iex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.adjascent_neighbors(Board.Point.new(0, 0)) [ %Board.Point{x: 0, y: 1}, %Board.Point{x: 1, y: 0} ] iex&gt; board = Board.new(Board.Size.new(3, 3)) iex&gt; board |&gt; Board.adjascent_neighbors(Board.Point.new(2, 2)) [ %Board.Point{x: 2, y: 1}, %Board.Point{x: 1, y: 2} ]","ref":"Snek.Board.html#adjascent_neighbors/2","title":"Snek.Board.adjascent_neighbors/2","type":"function"},{"doc":"Returns the number of snakes on the board who are still alive (not eliminated).Examplesiex&gt; apple = Board.Point.new(1, 4) iex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 5)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(apple) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, ids_and_heads) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board3 = Board.maybe_feed_snakes(board2) iex&gt; board4 = Board.move_snakes(board3, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board5 = Board.maybe_eliminate_snakes(board4) iex&gt; Board.alive_snakes_remaining(board5) 1","ref":"Snek.Board.html#alive_snakes_remaining/1","title":"Snek.Board.alive_snakes_remaining/1","type":"function"},{"doc":"Returns a list of all even points on the board, alternating like a checkerboard.Examplesiex&gt; Board.new(Board.Size.new(3, 3)) |&gt; Board.all_even_points [ %Board.Point{x: 0, y: 0}, %Board.Point{x: 0, y: 2}, %Board.Point{x: 1, y: 1}, %Board.Point{x: 2, y: 0}, %Board.Point{x: 2, y: 2} ]","ref":"Snek.Board.html#all_even_points/1","title":"Snek.Board.all_even_points/1","type":"function"},{"doc":"Returns a list of all points on the board.Examplesiex&gt; Board.new(Board.Size.new(2, 2)) |&gt; Board.all_points [ %Board.Point{x: 0, y: 0}, %Board.Point{x: 0, y: 1}, %Board.Point{x: 1, y: 0}, %Board.Point{x: 1, y: 1} ]","ref":"Snek.Board.html#all_points/1","title":"Snek.Board.all_points/1","type":"function"},{"doc":"Returns true if and only if any of the given points on the board are occupied.A point may be occupied by an apple, or any snake's body part.Examplesiex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.any_points_occupied?([Board.Point.new(1, 3), Board.Point.new(0, 0)]) false iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(Board.Point.new(1, 3)) iex&gt; board |&gt; Board.any_points_occupied?([Board.Point.new(1, 3), Board.Point.new(0, 0)]) true","ref":"Snek.Board.html#any_points_occupied?/2","title":"Snek.Board.any_points_occupied?/2","type":"function"},{"doc":"Returns the point at the center of the board.If the board width or height are even, the center will be offset because boards are a discrete grid.Examplesiex&gt; Board.new(Board.Size.new(3, 3)) |&gt; Board.center_point() %Board.Point{x: 1, y: 1} iex&gt; Board.new(Board.Size.new(8, 8)) |&gt; Board.center_point() %Board.Point{x: 3, y: 3}","ref":"Snek.Board.html#center_point/1","title":"Snek.Board.center_point/1","type":"function"},{"doc":"Returns a list of neighboring points diagonal to a point of origin.This excludes points that are outside of the board's boundaries.Examplesiex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.diagonal_neighbors(Board.Point.new(1, 1)) [ %Board.Point{x: 0, y: 0}, %Board.Point{x: 2, y: 0}, %Board.Point{x: 2, y: 2}, %Board.Point{x: 0, y: 2} ] iex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.diagonal_neighbors(Board.Point.new(0, 0)) [%Board.Point{x: 1, y: 1}] iex&gt; board = Board.new(Board.Size.new(3, 3)) iex&gt; board |&gt; Board.diagonal_neighbors(Board.Point.new(2, 2)) [%Board.Point{x: 1, y: 1}]","ref":"Snek.Board.html#diagonal_neighbors/2","title":"Snek.Board.diagonal_neighbors/2","type":"function"},{"doc":"Returns true if and only if this board is empty, otherwise false.The board is considered empty if it does not contain any snakes or apples.Examplesiex&gt; Board.new(Board.Size.small) |&gt; Board.empty? true iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple_at_center iex&gt; Board.empty?(board) false iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake_at_center(&quot;mysnek&quot;) iex&gt; Board.empty?(board) false","ref":"Snek.Board.html#empty?/1","title":"Snek.Board.empty?/1","type":"function"},{"doc":"Eliminate this snake if it has moved out of bounds, collided with itself, collided with another snake body, or lost in a head-to-head collision.If the snake is already previously eliminated, it will be returned unchanged regardless of any new collisions.Pass the snakes_by_length_descending argument as an ordered list of all snakes such that ambiguous collisions will be tied by snakes which appear first in the list. For example, if longer snakes should be considered first, pass a list of all snakes ordered by their respective lengths descending.Examplesiex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 3)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_snakes(ids_and_heads) iex&gt; board1 = Board.move_snakes(board0, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :east}]) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :east}]) iex&gt; snek0 = board2.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek0_eliminated = Board.maybe_eliminate_snake(board2, snek0, board2.snakes) iex&gt; snek0_eliminated.state {:eliminated, :collision, &quot;snek1&quot;} iex&gt; snek0_double_eliminated = Board.maybe_eliminate_snake(board2, snek0_eliminated, board2.snakes) iex&gt; snek0_double_eliminated == snek0_eliminated true iex&gt; start_length = 3 iex&gt; start_health = 1 iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_snake(&quot;snek0&quot;, Board.Point.new(1, 1), start_length, start_health) iex&gt; board1 = Board.reduce_snake_healths(board0) iex&gt; [snek0 | _] = board1.snakes iex&gt; snek0_eliminated = Board.maybe_eliminate_snake(board1, snek0, board1.snakes) iex&gt; snek0_eliminated.state {:eliminated, :starvation}","ref":"Snek.Board.html#maybe_eliminate_snake/3","title":"Snek.Board.maybe_eliminate_snake/3","type":"function"},{"doc":"Eliminate snakes who have moved out of bounds, collided with themselves, collided with other snake bodies, or lost in a head-to-head collision.Eliminations are decided by maybe_eliminate_snake/3 for each snake, giving priority to longer snakes in in ambiguous collisions.Examplesiex&gt; apple = Board.Point.new(1, 4) iex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 5)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(apple) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, ids_and_heads) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board3 = Board.maybe_feed_snakes(board2) iex&gt; board4 = Board.move_snakes(board3, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board5 = Board.maybe_eliminate_snakes(board4) iex&gt; board6 = Board.reduce_snake_healths(board5) iex&gt; snek0 = board6.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek1 = board6.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek1&quot;)) iex&gt; snek0.state {:eliminated, :head_to_head, &quot;snek1&quot;} iex&gt; snek1.state :alive","ref":"Snek.Board.html#maybe_eliminate_snakes/1","title":"Snek.Board.maybe_eliminate_snakes/1","type":"function"},{"doc":"Feed snakes who eat an apple.For all apples on the board, if any snake eats it, remove the apple from the board and feed each snake who ate it.A snake eats an apple if the snake's head is at the same position as the apple, and the snake is alive (not eliminated), and the snake has at least one body part.Feeding a snake is defined by Snek.Board.Snake.feed/2.Returns the modified board state.Examplesiex&gt; apple = Board.Point.new(1, 4) iex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 5)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(apple) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, ids_and_heads) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board3 = Board.maybe_feed_snakes(board2) iex&gt; snek0 = board3.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek1 = board3.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek1&quot;)) iex&gt; length(snek0.body) 3 iex&gt; length(snek1.body) 4 iex&gt; apple = Board.Point.new(1, 4) iex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 5)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(apple) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, ids_and_heads) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :east}]) iex&gt; board3 = Board.maybe_feed_snakes(board2) iex&gt; snek0 = board3.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek1 = board3.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek1&quot;)) iex&gt; length(snek0.body) 3 iex&gt; length(snek1.body) 3 iex&gt; board3 == board2 true","ref":"Snek.Board.html#maybe_feed_snakes/1","title":"Snek.Board.maybe_feed_snakes/1","type":"function"},{"doc":"Moves a snake on the board according to its move for this turn.A snake moves by slithering by one space per turn, in other words stepping in one direction by adding a new head part and removing a tail part.If nil is provided as the move, the snake will by default continue moving in the last moved direction. If the snake has not yet moved at all since spawning, it will default to moving :north.Returns a board with this snake's move applied.Examplesiex&gt; board0 = Board.new(Board.Size.small) iex&gt; {:ok, board1} = Board.spawn_snake(board0, &quot;snek0&quot;, Board.Point.new(1, 1)) iex&gt; board2 = Board.move_snake(board1, &quot;snek0&quot;, :east) iex&gt; board2.snakes [ %Board.Snake{ body: [%Board.Point{x: 2, y: 1}, %Board.Point{x: 1, y: 1}, %Board.Point{x: 1, y: 1}], state: :alive, health: 100, id: &quot;snek0&quot; } ]","ref":"Snek.Board.html#move_snake/3","title":"Snek.Board.move_snake/3","type":"function"},{"doc":"Moves each snake on the board according to their respective moves for this turn.Snakes move by slithering by one space per turn, in other words stepping in one direction by adding a new head part and removing a tail part.If nil is provided as a move, the snake will by default continue moving in the last moved direction. If the snake has not yet moved at all since spawning, it will default to moving :north.Returns a board with all moves applied.Examplesiex&gt; board0 = Board.new(Board.Size.small) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(5, 5)}]) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :east}, {&quot;snek1&quot;, nil}]) iex&gt; board2.snakes [ %Board.Snake{ body: [%Board.Point{x: 5, y: 4}, %Board.Point{x: 5, y: 5}, %Board.Point{x: 5, y: 5}], state: :alive, health: 100, id: &quot;snek1&quot; }, %Board.Snake{ body: [%Board.Point{x: 2, y: 1}, %Board.Point{x: 1, y: 1}, %Board.Point{x: 1, y: 1}], state: :alive, health: 100, id: &quot;snek0&quot; } ]","ref":"Snek.Board.html#move_snakes/2","title":"Snek.Board.move_snakes/2","type":"function"},{"doc":"Returns a new empty board of a given size.Examplesiex&gt; Board.new(Board.Size.small) %Board{size: %Board.Size{width: 7, height: 7}, apples: [], snakes: []}","ref":"Snek.Board.html#new/1","title":"Snek.Board.new/1","type":"function"},{"doc":"Returns true if and only if the given point on the board is occupied, otherwise false.A point may be occupied by an apple, or any snake's body part.Examplesiex&gt; Board.new(Board.Size.small) |&gt; Board.occupied?(Board.Point.new(1, 3)) false iex&gt; point = Board.Point.new(1, 3) iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(point) iex&gt; Board.occupied?(board, point) true iex&gt; point = Board.Point.new(1, 3) iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake(&quot;mysnek&quot;, point) iex&gt; Board.occupied?(board, point) true","ref":"Snek.Board.html#occupied?/2","title":"Snek.Board.occupied?/2","type":"function"},{"doc":"Returns true if and only if the given point on the board is occupied by an apple, otherwise false.Examplesiex&gt; Board.new(Board.Size.small) |&gt; Board.occupied_by_apple?(Board.Point.new(1, 3)) false iex&gt; point = Board.Point.new(1, 3) iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(point) iex&gt; Board.occupied_by_apple?(board, point) true iex&gt; point = Board.Point.new(1, 3) iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake(&quot;mysnek&quot;, point) iex&gt; Board.occupied_by_apple?(board, point) false","ref":"Snek.Board.html#occupied_by_apple?/2","title":"Snek.Board.occupied_by_apple?/2","type":"function"},{"doc":"Returns true if and only if the given point on the board is occupied by a snake's body part, otherwise false.Examplesiex&gt; Board.new(Board.Size.small) |&gt; Board.occupied_by_snake?(Board.Point.new(1, 3)) false iex&gt; point = Board.Point.new(1, 3) iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(point) iex&gt; Board.occupied_by_snake?(board, point) false iex&gt; point = Board.Point.new(1, 3) iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake(&quot;mysnek&quot;, point) iex&gt; Board.occupied_by_snake?(board, point) true","ref":"Snek.Board.html#occupied_by_snake?/2","title":"Snek.Board.occupied_by_snake?/2","type":"function"},{"doc":"Returns a list of all occupied points on the board.Examplesiex&gt; apple = Board.Point.new(0, 1) iex&gt; {:ok, board} = Board.new(Board.Size.new(2, 2)) |&gt; Board.spawn_apple(apple) iex&gt; Board.occupied_points(board) [ %Board.Point{x: 0, y: 1} ]","ref":"Snek.Board.html#occupied_points/1","title":"Snek.Board.occupied_points/1","type":"function"},{"doc":"Returns true if and only if this point is outside of the board's boundaries, in other words the opposite of within_bounds?/2.Examplesiex&gt; board = Board.new(Board.Size.new(3, 3)) iex&gt; board |&gt; Board.out_of_bounds?(Board.Point.new(0, 0)) false iex&gt; board |&gt; Board.out_of_bounds?(Board.Point.new(1, 2)) false iex&gt; board |&gt; Board.out_of_bounds?(Board.Point.new(-1, 0)) true iex&gt; board |&gt; Board.out_of_bounds?(Board.Point.new(0, 3)) true","ref":"Snek.Board.html#out_of_bounds?/2","title":"Snek.Board.out_of_bounds?/2","type":"function"},{"doc":"Reduce the health of each snake by one point.Does not affect the health of eliminated snakes.Returns a board with all snake health reductions applied.Examplesiex&gt; apple = Board.Point.new(1, 4) iex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 5)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(apple) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, ids_and_heads) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board3 = Board.maybe_feed_snakes(board2) iex&gt; board4 = Board.move_snakes(board3, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board5 = Board.maybe_eliminate_snakes(board4) iex&gt; board6 = Board.reduce_snake_healths(board5) iex&gt; snek0 = board6.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek1 = board6.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek1&quot;)) iex&gt; snek0.health # Eliminated before reducing health 100 iex&gt; snek1.health # Not eliminiated 99","ref":"Snek.Board.html#reduce_snake_healths/1","title":"Snek.Board.reduce_snake_healths/1","type":"function"},{"doc":"Returns true if and only if snake_a's head is in collision with any of snake_b's body parts, excluding snake_b's head. Otherwise, returns false.The two snake arguments commutative. One snake my collide with another snake's body, and yet the other snake's head may not be in a collision.As such, head-to-head collisions are not detected this way. For that, use snake_loses_head_to_head_collision?/2 instead.Examplesiex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 3)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_snakes(ids_and_heads) iex&gt; board1 = Board.move_snakes(board0, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :east}]) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :east}]) iex&gt; snek0 = board2.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek1 = board2.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek1&quot;)) iex&gt; Board.snake_collides_with_other_snake?(snek0, snek1) true iex&gt; Board.snake_collides_with_other_snake?(snek1, snek0) false","ref":"Snek.Board.html#snake_collides_with_other_snake?/2","title":"Snek.Board.snake_collides_with_other_snake?/2","type":"function"},{"doc":"Returns true if and only if there is a head-to-head collision between snake_a and snake_b and snake_a's body length is shorter or equal to snake_b's body length, thereby causing snake_a to lose the head-to-head.Examplesiex&gt; apple = Board.Point.new(1, 4) iex&gt; ids_and_heads = [{&quot;snek0&quot;, Board.Point.new(1, 1)}, {&quot;snek1&quot;, Board.Point.new(1, 5)}] iex&gt; {:ok, board0} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(apple) iex&gt; {:ok, board1} = Board.spawn_snakes(board0, ids_and_heads) iex&gt; board2 = Board.move_snakes(board1, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; board3 = Board.maybe_feed_snakes(board2) iex&gt; board4 = Board.move_snakes(board3, [{&quot;snek0&quot;, :south}, {&quot;snek1&quot;, :north}]) iex&gt; snek0 = board4.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek0&quot;)) iex&gt; snek1 = board4.snakes |&gt; Enum.find(&amp;(&amp;1.id == &quot;snek1&quot;)) iex&gt; Board.snake_loses_head_to_head_collision?(snek0, snek1) true iex&gt; Board.snake_loses_head_to_head_collision?(snek1, snek0) false","ref":"Snek.Board.html#snake_loses_head_to_head_collision?/2","title":"Snek.Board.snake_loses_head_to_head_collision?/2","type":"function"},{"doc":"Returns true if and only if this snake has some body part outside of the board's boundaries.Examplesiex&gt; {:ok, board} = Board.new(Board.Size.new(3, 3)) |&gt; Board.spawn_snake(&quot;mysnek&quot;, Board.Point.new(1, 1)) iex&gt; [snake | _] = board.snakes iex&gt; Board.snake_out_of_bounds?(board, snake) false iex&gt; {:ok, board} = Board.new(Board.Size.new(3, 3)) |&gt; Board.spawn_snake(&quot;mysnek&quot;, Board.Point.new(0, 3)) iex&gt; [snake | _] = board.snakes iex&gt; Board.snake_out_of_bounds?(board, snake) true","ref":"Snek.Board.html#snake_out_of_bounds?/2","title":"Snek.Board.snake_out_of_bounds?/2","type":"function"},{"doc":"Spawns an apple at the specified point on the board.Returns {:ok, board} if there is space available, returns {:error, :occupied} otherwise.Examplesiex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(Board.Point.new(1, 1)) iex&gt; board %Board{ apples: [%Board.Point{x: 1, y: 1}], size: %Board.Size{height: 7, width: 7}, snakes: [] } iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(Board.Point.new(1, 1)) iex&gt; board |&gt; Board.spawn_apple(Board.Point.new(1, 1)) {:error, :occupied}","ref":"Snek.Board.html#spawn_apple/2","title":"Snek.Board.spawn_apple/2","type":"function"},{"doc":"Spawns an apple in the center of the board.Returns {:ok, board} if there is space available, returns {:error, :occupied} otherwise.Examplesiex&gt; {:ok, board} = Board.new(Board.Size.new(3, 3)) |&gt; Board.spawn_apple_at_center() iex&gt; board %Board{ apples: [%Board.Point{x: 1, y: 1}], size: %Board.Size{height: 3, width: 3}, snakes: [] } iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple_at_center() iex&gt; board |&gt; Board.spawn_apple_at_center() {:error, :occupied}","ref":"Snek.Board.html#spawn_apple_at_center/1","title":"Snek.Board.spawn_apple_at_center/1","type":"function"},{"doc":"Spawns an apple at the specified point on the board.Unlike spawn_apple/2 this function will not check whether there is space available. You are expected to only use this function if you are otherwise performing that validation yourself. For example, it may be more efficient to precompute available spaces before spawning many apples.Returns a board state with the apple added.Examplesiex&gt; board = Board.new(Board.Size.small) |&gt; Board.spawn_apple_unchecked(Board.Point.new(1, 1)) iex&gt; board %Board{ apples: [%Board.Point{x: 1, y: 1}], size: %Board.Size{height: 7, width: 7}, snakes: [] }","ref":"Snek.Board.html#spawn_apple_unchecked/2","title":"Snek.Board.spawn_apple_unchecked/2","type":"function"},{"doc":"Spawns apples at each of the specified points on the board.Returns {:ok, board} if there is space available, returns {:error, :occupied} otherwise.Examplesiex&gt; points = [Board.Point.new(1, 1), Board.Point.new(1, 2)] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apples(points) iex&gt; board %Board{ apples: [ %Board.Point{x: 1, y: 1}, %Snek.Board.Point{x: 1, y: 2} ], size: %Snek.Board.Size{height: 7, width: 7}, snakes: [] } iex&gt; occupied_point = Board.Point.new(1, 1) iex&gt; new_points = [occupied_point, Board.Point.new(1, 2)] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(occupied_point) iex&gt; Board.spawn_apples(board, new_points) {:error, :occupied}","ref":"Snek.Board.html#spawn_apples/2","title":"Snek.Board.spawn_apples/2","type":"function"},{"doc":"Spawns a snake at the specified point on the board.Returns {:ok, board} if there is space available, returns {:error, :occupied} otherwise.Examplesiex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake(&quot;mysnek&quot;, Board.Point.new(1, 1)) iex&gt; board.snakes [ %Board.Snake{ body: [%Board.Point{x: 1, y: 1}, %Board.Point{x: 1, y: 1}, %Board.Point{x: 1, y: 1}], state: :alive, health: 100, id: &quot;mysnek&quot; } ] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake(&quot;mysnek&quot;, Board.Point.new(1, 1)) iex&gt; Board.spawn_snake(board, &quot;mysnek&quot;, Board.Point.new(1, 1)) {:error, :occupied}","ref":"Snek.Board.html#spawn_snake/5","title":"Snek.Board.spawn_snake/5","type":"function"},{"doc":"Spawns a snake in the center of the board.Returns {:ok, board} if there is space available, returns {:error, :occupied} otherwise.Examplesiex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake_at_center(&quot;mysnek&quot;) iex&gt; board.snakes [ %Board.Snake{ body: [%Board.Point{x: 3, y: 3}, %Board.Point{x: 3, y: 3}, %Board.Point{x: 3, y: 3}], state: :alive, health: 100, id: &quot;mysnek&quot; } ] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snake_at_center(&quot;mysnek&quot;) iex&gt; Board.spawn_snake_at_center(board, &quot;mysnek&quot;) {:error, :occupied}","ref":"Snek.Board.html#spawn_snake_at_center/4","title":"Snek.Board.spawn_snake_at_center/4","type":"function"},{"doc":"Spawns multiple snakes, each at a specified point on the board.Returns {:ok, board} if there is space available, returns {:error, :occupied} otherwise.Examplesiex&gt; ids_and_heads = [{&quot;snek1&quot;, Board.Point.new(1, 1)}, {&quot;snek2&quot;, Board.Point.new(5, 5)}] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_snakes(ids_and_heads) iex&gt; board.snakes [ %Board.Snake{ body: [%Board.Point{x: 5, y: 5}, %Board.Point{x: 5, y: 5}, %Board.Point{x: 5, y: 5}], state: :alive, health: 100, id: &quot;snek2&quot; }, %Board.Snake{ body: [%Board.Point{x: 1, y: 1}, %Board.Point{x: 1, y: 1}, %Board.Point{x: 1, y: 1}], state: :alive, health: 100, id: &quot;snek1&quot; } ] iex&gt; ids_and_heads = [{&quot;snek1&quot;, Board.Point.new(1, 1)}, {&quot;snek2&quot;, Board.Point.new(1, 1)}] iex&gt; Board.new(Board.Size.small) |&gt; Board.spawn_snakes(ids_and_heads) {:error, :occupied}","ref":"Snek.Board.html#spawn_snakes/4","title":"Snek.Board.spawn_snakes/4","type":"function"},{"doc":"Returns a list of unoccupied neighboring points adjascent to a point of origin.This excludes any points occupied by an apple, or any snake's body part.This excludes points that are outside of the board's boundaries.Examplesiex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.unoccupied_adjascent_neighbors(Board.Point.new(1, 1)) [ %Board.Point{x: 1, y: 0}, %Board.Point{x: 1, y: 2}, %Board.Point{x: 2, y: 1}, %Board.Point{x: 0, y: 1} ] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(Board.Point.new(1, 2)) iex&gt; board |&gt; Board.unoccupied_adjascent_neighbors(Board.Point.new(1, 1)) [ %Board.Point{x: 1, y: 0}, %Board.Point{x: 2, y: 1}, %Board.Point{x: 0, y: 1} ]","ref":"Snek.Board.html#unoccupied_adjascent_neighbors/2","title":"Snek.Board.unoccupied_adjascent_neighbors/2","type":"function"},{"doc":"Returns a list of unoccupied neighboring points diagonal to a point of origin.This excludes any points occupied by an apple, or any snake's body part.This excludes points that are outside of the board's boundaries.Examplesiex&gt; board = Board.new(Board.Size.small) iex&gt; board |&gt; Board.unoccupied_diagonal_neighbors(Board.Point.new(1, 1)) [ %Board.Point{x: 0, y: 0}, %Board.Point{x: 2, y: 0}, %Board.Point{x: 2, y: 2}, %Board.Point{x: 0, y: 2} ] iex&gt; {:ok, board} = Board.new(Board.Size.small) |&gt; Board.spawn_apple(Board.Point.new(0, 0)) iex&gt; board |&gt; Board.unoccupied_diagonal_neighbors(Board.Point.new(1, 1)) [ %Board.Point{x: 2, y: 0}, %Board.Point{x: 2, y: 2}, %Board.Point{x: 0, y: 2} ]","ref":"Snek.Board.html#unoccupied_diagonal_neighbors/2","title":"Snek.Board.unoccupied_diagonal_neighbors/2","type":"function"},{"doc":"Returns a list of all unoccupied points on the board.Examplesiex&gt; apple = Board.Point.new(0, 1) iex&gt; {:ok, board} = Board.new(Board.Size.new(2, 2)) |&gt; Board.spawn_apple(apple) iex&gt; Board.unoccupied_points(board) [ %Board.Point{x: 0, y: 0}, %Board.Point{x: 1, y: 0}, %Board.Point{x: 1, y: 1} ]","ref":"Snek.Board.html#unoccupied_points/1","title":"Snek.Board.unoccupied_points/1","type":"function"},{"doc":"Returns true if and only if this point is within the board's boundaries, otherwise false.Examplesiex&gt; board = Board.new(Board.Size.new(3, 3)) iex&gt; board |&gt; Board.within_bounds?(Board.Point.new(0, 0)) true iex&gt; board |&gt; Board.within_bounds?(Board.Point.new(1, 2)) true iex&gt; board |&gt; Board.within_bounds?(Board.Point.new(-1, 0)) false iex&gt; board |&gt; Board.within_bounds?(Board.Point.new(0, 3)) false","ref":"Snek.Board.html#within_bounds?/2","title":"Snek.Board.within_bounds?/2","type":"function"},{"doc":"When spawning, {:ok, board} if there is space available, {:error, :occupied} otherwise.","ref":"Snek.Board.html#t:spawn_result/0","title":"Snek.Board.spawn_result/0","type":"type"},{"doc":"A board position.","ref":"Snek.Board.html#t:t/0","title":"Snek.Board.t/0","type":"type"},{"doc":"A struct for representing points on a board's grid.","ref":"Snek.Board.Point.html","title":"Snek.Board.Point","type":"module"},{"doc":"Returns a list of neighboring points adjascent to a point of origin.Examplesiex&gt; Point.adjascent_neighbors(Point.new(1, 1)) [ %Point{x: 1, y: 0}, %Point{x: 1, y: 2}, %Point{x: 2, y: 1}, %Point{x: 0, y: 1} ] iex&gt; Point.adjascent_neighbors(Point.new(0, 0)) [ %Point{x: 0, y: -1}, %Point{x: 0, y: 1}, %Point{x: 1, y: 0}, %Point{x: -1, y: 0} ]","ref":"Snek.Board.Point.html#adjascent_neighbors/1","title":"Snek.Board.Point.adjascent_neighbors/1","type":"function"},{"doc":"Returns a list of neighboring points diagonal to a point of origin.Examplesiex&gt; Point.diagonal_neighbors(Point.new(1, 1)) [ %Point{x: 0, y: 0}, %Point{x: 2, y: 0}, %Point{x: 2, y: 2}, %Point{x: 0, y: 2} ] iex&gt; Point.diagonal_neighbors(Point.new(0, 0)) [ %Point{x: -1, y: -1}, %Point{x: 1, y: -1}, %Point{x: 1, y: 1}, %Point{x: -1, y: 1} ]","ref":"Snek.Board.Point.html#diagonal_neighbors/1","title":"Snek.Board.Point.diagonal_neighbors/1","type":"function"},{"doc":"Returns the difference between two points, which could be used to find a vector between points, such as when using the neck and head of a snake to determine the point continuing in the last moved direction.Examplesiex&gt; Point.difference(Point.new(1, 2), Point.new(1, 3)) %Point{x: 0, y: -1} iex&gt; Point.difference(Point.new(4, 4), Point.new(5, 4)) %Point{x: -1, y: 0}","ref":"Snek.Board.Point.html#difference/2","title":"Snek.Board.Point.difference/2","type":"function"},{"doc":"Returns true if and only if this point falls on an even square for an board, alternating like a checkerboard.Examplesiex&gt; Point.even?(Point.new(0, 0)) true iex&gt; Point.even?(Point.new(0, 1)) false iex&gt; Point.even?(Point.new(0, 2)) true iex&gt; Point.even?(Point.new(1, 0)) false iex&gt; Point.even?(Point.new(1, 1)) true iex&gt; Point.even?(Point.new(1, 2)) false","ref":"Snek.Board.Point.html#even?/1","title":"Snek.Board.Point.even?/1","type":"function"},{"doc":"Returns the Manhattan distance between two points.Examplesiex&gt; Point.manhattan_distance(Point.new(0, 0), Point.new(1, 2)) 3","ref":"Snek.Board.Point.html#manhattan_distance/2","title":"Snek.Board.Point.manhattan_distance/2","type":"function"},{"doc":"Returns a new point at the given X and Y coordinates.Examplesiex&gt; Point.new(0, 0) %Point{x: 0, y: 0} iex&gt; Point.new(3, 1) %Point{x: 3, y: 1} iex&gt; Point.new(-2, 0) %Point{x: -2, y: 0}","ref":"Snek.Board.Point.html#new/2","title":"Snek.Board.Point.new/2","type":"function"},{"doc":"Rotates a point 90 degrees clockwise.This is useful for rotating vectors, which can help with relative directions such as :left and :right.Examplesiex&gt; Point.rotate_clockwise(Point.new(0, 1)) %Point{x: -1, y: 0}","ref":"Snek.Board.Point.html#rotate_clockwise/1","title":"Snek.Board.Point.rotate_clockwise/1","type":"function"},{"doc":"Rotates a point 90 degrees counter-clockwise.This is useful for rotating vectors, which can help with relative directions such as :left and :right.Examplesiex&gt; Point.rotate_counterclockwise(Point.new(-1, 0)) %Point{x: 0, y: 1}","ref":"Snek.Board.Point.html#rotate_counterclockwise/1","title":"Snek.Board.Point.rotate_counterclockwise/1","type":"function"},{"doc":"Returns the point that is one step toward a given direction from a point of origin.Examplesiex&gt; Point.new(5, 5) |&gt; Point.step(:north) %Point{x: 5, y: 4} iex&gt; Point.new(5, 5) |&gt; Point.step(:south) %Point{x: 5, y: 6} iex&gt; Point.new(5, 5) |&gt; Point.step(:east) %Point{x: 6, y: 5} iex&gt; Point.new(5, 5) |&gt; Point.step(:west) %Point{x: 4, y: 5} iex&gt; Point.new(5, 5) |&gt; Point.step(:northwest) %Point{x: 4, y: 4} iex&gt; Point.new(5, 5) |&gt; Point.step(:northeast) %Point{x: 6, y: 4} iex&gt; Point.new(5, 5) |&gt; Point.step(:southeast) %Point{x: 6, y: 6} iex&gt; Point.new(5, 5) |&gt; Point.step(:southwest) %Point{x: 4, y: 6}","ref":"Snek.Board.Point.html#step/2","title":"Snek.Board.Point.step/2","type":"function"},{"doc":"Returns the sum of two points, which could be used to apply a vector point to a fixed points, such as when using the neck and head of a snake to determine the point continuing in the last moved direction.Examplesiex&gt; Point.sum(Point.new(1, 2), Point.new(1, 0)) %Point{x: 2, y: 2} iex&gt; Point.sum(Point.new(4, 4), Point.new(-1, 1)) %Point{x: 3, y: 5}","ref":"Snek.Board.Point.html#sum/2","title":"Snek.Board.Point.sum/2","type":"function"},{"doc":"Returns true if and only if both X and Y are zero, which could be used to determine if a point is a null vector.Examplesiex&gt; Point.zero?(Point.new(0, 0)) true iex&gt; Point.zero?(Point.new(0, 1)) false","ref":"Snek.Board.Point.html#zero?/1","title":"Snek.Board.Point.zero?/1","type":"function"},{"doc":"A direction from a point toward its adjascent or diagonal neighbor.","ref":"Snek.Board.Point.html#t:direction/0","title":"Snek.Board.Point.direction/0","type":"type"},{"doc":"A point on a board.May be relative or absolute.","ref":"Snek.Board.Point.html#t:t/0","title":"Snek.Board.Point.t/0","type":"type"},{"doc":"A point's X coordinate.Smaller values are toward the west side of the board, larger are toward the east.For absolute coordinates on a board, use an integer between zero and the board width minus one.For relative points, you may use a negative integer.","ref":"Snek.Board.Point.html#t:x/0","title":"Snek.Board.Point.x/0","type":"type"},{"doc":"A point's Y coordinate.Smaller values are toward the north side of the board, larger are toward the south.For absolute coordinates on a board, use an integer between zero and the board height minus one.For relative points, you may use a negative integer.","ref":"Snek.Board.Point.html#t:y/0","title":"Snek.Board.Point.y/0","type":"type"},{"doc":"A struct representing the size of a game board.A board is always rectangular (or square), and is represented by a width and a height.Arbitrary board sizes may be created with new/2.There are some helpers functions for some suggested board sizes, including small/0, medium/0, and large/0. These suggestions are based on the default board sizes in Battlesnake.","ref":"Snek.Board.Size.html","title":"Snek.Board.Size","type":"module"},{"doc":"Return a large (19x19) board size.Examplesiex&gt; Size.large %Size{width: 19, height: 19}","ref":"Snek.Board.Size.html#large/0","title":"Snek.Board.Size.large/0","type":"function"},{"doc":"Return a medium (11x11) board size.Examplesiex&gt; Size.medium %Size{width: 11, height: 11}","ref":"Snek.Board.Size.html#medium/0","title":"Snek.Board.Size.medium/0","type":"function"},{"doc":"Returns a board size of the specified width and height.","ref":"Snek.Board.Size.html#new/2","title":"Snek.Board.Size.new/2","type":"function"},{"doc":"Return a small (7x7) board size.Examplesiex&gt; Size.small %Size{width: 7, height: 7}","ref":"Snek.Board.Size.html#small/0","title":"Snek.Board.Size.small/0","type":"function"},{"doc":"","ref":"Snek.Board.Size.html#t:t/0","title":"Snek.Board.Size.t/0","type":"type"},{"doc":"Represents a snake on a board.You may also refer to it as a &quot;snake on a plane&quot;, as the joke goes in the Battlesnake community. 😎","ref":"Snek.Board.Snake.html","title":"Snek.Board.Snake","type":"module"},{"doc":"Returns true if and only if the snake is alive (not eliminated).This does not check whether the snake's elimination status should be changed, it is just a helper to check current state.Examplesiex&gt; body = [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2)] iex&gt; Snake.alive?(%Snake{id: &quot;mysnek&quot;, state: :alive, health: 98, body: body}) true iex&gt; body = [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2)] iex&gt; Snake.alive?(%Snake{id: &quot;mysnek&quot;, state: {:eliminated, :starvation}, health: 0, body: body}) false","ref":"Snek.Board.Snake.html#alive?/1","title":"Snek.Board.Snake.alive?/1","type":"function"},{"doc":"Returns true if and only if the snake is eliminated.This does not check whether the snake's elimination status should be changed, it is just a helper to check current state.Examplesiex&gt; body = [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2)] iex&gt; Snake.eliminated?(%Snake{id: &quot;mysnek&quot;, state: :alive, health: 98, body: body}) false iex&gt; body = [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2)] iex&gt; Snake.eliminated?(%Snake{id: &quot;mysnek&quot;, state: {:eliminated, :starvation}, health: 0, body: body}) true","ref":"Snek.Board.Snake.html#eliminated?/1","title":"Snek.Board.Snake.eliminated?/1","type":"function"},{"doc":"Feed a snake and grow its tail.A snake is fed by restoring its health to a given value, and adding a part to its tail. The new tail part is added in the same position as the current tail (the last body part). Tail body parts may overlap until the snake moves.Returns the modified snake.Examplesiex&gt; body = [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 98, body: body} iex&gt; Snake.feed(snake, 100) %Snake{ id: &quot;mysnek&quot;, state: :alive, health: 100, body: [ Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2), Snek.Board.Point.new(1, 2) ] }","ref":"Snek.Board.Snake.html#feed/2","title":"Snek.Board.Snake.feed/2","type":"function"},{"doc":"Grow a snake's tail.Adds a part to the snake's tail. The new tail part is added in the same position as the current tail (the last body part). Tail body parts may overlap until the snake moves.This is equivelent to the tail growth in feed/2 but without affecting the snake's health.Returns the modified snake.Examplesiex&gt; body = [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 98, body: body} iex&gt; Snake.grow(snake) %Snake{ id: &quot;mysnek&quot;, state: :alive, health: 98, body: [ Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 2), Snek.Board.Point.new(1, 2) ] } iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: []} iex&gt; snake == Snake.grow(snake) true","ref":"Snek.Board.Snake.html#grow/1","title":"Snek.Board.Snake.grow/1","type":"function"},{"doc":"Returns the head of a snake.If the snake has at least one body part, the first body part (the head) is returned. Otherwise, nil is returned.Examplesiex&gt; body = [Snek.Board.Point.new(1, 2), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: body} iex&gt; Snake.head(snake) %Snek.Board.Point{x: 1, y: 2} iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: []} iex&gt; Snake.head(snake) nil","ref":"Snek.Board.Snake.html#head/1","title":"Snek.Board.Snake.head/1","type":"function"},{"doc":"Decrements the snake's health by 1 point.Returns the modified snake.Examplesiex&gt; body = List.duplicate(Snek.Board.Point.new(1, 1), 3) iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: body} iex&gt; Snake.hurt(snake).health 99","ref":"Snek.Board.Snake.html#hurt/1","title":"Snek.Board.Snake.hurt/1","type":"function"},{"doc":"Moves the snake one space in a given direction.Moving consists of adding a body part to the head of the snake in the given direction, and also removing the tail body part. The snake's body length remains unchanged.If the snake is already eliminated or the snake does not have any body parts, no move will be applied and the snake will remain unchanged.If the direction given is :left, :right, or :forward then the snake is moved in that direction relative to the snake's last moved direction.If the direction given is nil, or not a valid direction in which to move, the snake will be moved in the :forward direction. If the snake does not have both head and neck body parts, or the head and neck are at teh same position, the snake will default to moving :north instead, as in that case the last moved direction cannot be extrapolated.Returns the modified snake.Examplesiex&gt; body = [Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: body} iex&gt; Snake.move(snake, :north) %Snake{ id: &quot;mysnek&quot;, state: :alive, health: 100, body: [Snek.Board.Point.new(1, 0), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] } iex&gt; body = [Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: {:eliminated, :starvation}, health: 0, body: body} iex&gt; snake == Snake.move(snake, :north) true iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: []} iex&gt; snake == Snake.move(snake, :north) true iex&gt; body = [Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: body} iex&gt; snake |&gt; Snake.move(:east) |&gt; Snake.move(nil) %Snake{ id: &quot;mysnek&quot;, state: :alive, health: 100, body: [Snek.Board.Point.new(3, 1), Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1)] } iex&gt; body = [Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;mysnek&quot;, state: :alive, health: 100, body: body} iex&gt; snake |&gt; Snake.move(:east) |&gt; Snake.move(:left) |&gt; Snake.move(:right) %Snake{ id: &quot;mysnek&quot;, state: :alive, health: 100, body: [Snek.Board.Point.new(3, 0), Snek.Board.Point.new(2, 0), Snek.Board.Point.new(2, 1)] }","ref":"Snek.Board.Snake.html#move/2","title":"Snek.Board.Snake.move/2","type":"function"},{"doc":"Returns the point that is one step toward a given direction from this snake's perspective.If the snake has no body parts, nil is returned instead of a point.If the direction given is :left, :right, or :forward then the point returned will be in that direction relative to the snake's last moved direction. If the snake does not have both head and neck body parts, or the head and neck are at the same position, then nil will be returned instead of a point, as in that case the last moved direction cannot be extrapolated.Examplesiex&gt; body = [Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 99, body: body} iex&gt; Snake.step(snake, :north) %Snek.Board.Point{x: 2, y: 0} iex&gt; body = [Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 99, body: body} iex&gt; Snake.step(snake, :left) %Snek.Board.Point{x: 2, y: 0} iex&gt; body = [Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 99, body: body} iex&gt; Snake.step(snake, :east) %Snek.Board.Point{x: 3, y: 1} iex&gt; body = [Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 99, body: body} iex&gt; Snake.step(snake, :forward) %Snek.Board.Point{x: 3, y: 1} iex&gt; body = [Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 99, body: body} iex&gt; Snake.step(snake, :right) %Snek.Board.Point{x: 2, y: 2} iex&gt; body = [Snek.Board.Point.new(2, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 99, body: body} iex&gt; Snake.step(snake, :south) %Snek.Board.Point{x: 2, y: 2} iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 0, body: []} iex&gt; Snake.step(snake, :south) nil iex&gt; body = [Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1), Snek.Board.Point.new(1, 1)] iex&gt; snake = %Snake{id: &quot;snek0&quot;, state: :alive, health: 100, body: body} iex&gt; Snake.step(snake, :forward) nil","ref":"Snek.Board.Snake.html#step/2","title":"Snek.Board.Snake.step/2","type":"function"},{"doc":"A unique ID to differentiate between snakes on a board","ref":"Snek.Board.Snake.html#t:id/0","title":"Snek.Board.Snake.id/0","type":"type"},{"doc":"A valid direction for a snake to move according to the game rules.","ref":"Snek.Board.Snake.html#t:snake_move/0","title":"Snek.Board.Snake.snake_move/0","type":"type"},{"doc":"Whether a snake is currently alive, or has been eliminated.If eliminated, the reason is encoded. If the elimination was caused by an opponent, the opponent's snake ID is also specified.","ref":"Snek.Board.Snake.html#t:state/0","title":"Snek.Board.Snake.state/0","type":"type"},{"doc":"A snake on a board.","ref":"Snek.Board.Snake.html#t:t/0","title":"Snek.Board.Snake.t/0","type":"type"},{"doc":"A behaviour module for implementing variations of game rules.Implementations define how a game plays out from start to finish, by dynamically specifying:init/2: The initial board positionc:next/2: Each next turn's board position after moves are applieddone?/1: When the game is considered over","ref":"Snek.Ruleset.html","title":"Snek.Ruleset","type":"behaviour"},{"doc":"Decide whether the game is over at this board position.","ref":"Snek.Ruleset.html#c:done?/1","title":"Snek.Ruleset.done?/1","type":"callback"},{"doc":"Decide the initial board position for a new game.","ref":"Snek.Ruleset.html#c:init/2","title":"Snek.Ruleset.init/2","type":"callback"},{"doc":"Apply moves and decide the next turn's board position.","ref":"Snek.Ruleset.html#c:next/3","title":"Snek.Ruleset.next/3","type":"callback"},{"doc":"Valid moves for a snake to play.","ref":"Snek.Ruleset.html#t:valid_move/0","title":"Snek.Ruleset.valid_move/0","type":"type"},{"doc":"The solo ruleset, based on the official Battlesnake solo rules.Solo rules are the same as Snek.Ruleset.Standard, except standard games end when there are fewer than 2 snakes remaining and solo games only end after the last remaining snake is eliminated.Effort is made to keep this implementation compatible with Battlesnake's official rules, so that it may be used for simulating game turns. If there is a mistake either in the implementation or the tests/specification, please report it as a bug.","ref":"Snek.Ruleset.Solo.html","title":"Snek.Ruleset.Solo","type":"module"},{"doc":"Callback implementation for Snek.Ruleset.done?/1.","ref":"Snek.Ruleset.Solo.html#done?/1","title":"Snek.Ruleset.Solo.done?/1","type":"function"},{"doc":"Callback implementation for Snek.Ruleset.init/2.","ref":"Snek.Ruleset.Solo.html#init/2","title":"Snek.Ruleset.Solo.init/2","type":"function"},{"doc":"Callback implementation for Snek.Ruleset.next/3.","ref":"Snek.Ruleset.Solo.html#next/3","title":"Snek.Ruleset.Solo.next/3","type":"function"},{"doc":"The standard ruleset, based on the official Battlesnake rules.Effort is made to keep this implementation compatible with Battlesnake's official rules, so that it may be used for simulating game turns. If there is a mistake either in the implementation or the tests/specification, please report it as a bug.","ref":"Snek.Ruleset.Standard.html","title":"Snek.Ruleset.Standard","type":"module"},{"doc":"Callback implementation for Snek.Ruleset.done?/1.","ref":"Snek.Ruleset.Standard.html#done?/1","title":"Snek.Ruleset.Standard.done?/1","type":"function"},{"doc":"Callback implementation for Snek.Ruleset.init/2.","ref":"Snek.Ruleset.Standard.html#init/2","title":"Snek.Ruleset.Standard.init/2","type":"function"},{"doc":"Callback implementation for Snek.Ruleset.next/3.","ref":"Snek.Ruleset.Standard.html#next/3","title":"Snek.Ruleset.Standard.next/3","type":"function"},{"doc":"SnekA framework for defining Battlesnake-compatible rulesets and board positions in Elixir.Battlesnake is an online competitive take on the classic game of Snake. Snek provides some structure and tooling that developers can use toward creating Battlesnake AIs in Elixir.This project is not affiliated with or sponsored by Battlesnake.","ref":"readme.html","title":"Snek","type":"extras"},{"doc":"This is an early work in progress, and should be considered experimental, incomplete, and unstable until v1.0.0, following Semantic Versioning.All notable changes will be recorded in the changelog.","ref":"readme.html#status","title":"Snek - Status","type":"extras"},{"doc":"Add snek to your list of dependencies in mix.exs:def deps do [ {:snek, &quot;~&gt; 0.2.0&quot;} ] end","ref":"readme.html#installation","title":"Snek - Installation","type":"extras"},{"doc":"Documentation can be found here at Hex.pm or generated from the source code using mix docs.","ref":"readme.html#documentation","title":"Snek - Documentation","type":"extras"},{"doc":"The following example simulates a couple of game turns using the Standard ruleset, for a two-player game, on a standard medium (11x11) board size.alias Snek.Ruleset.Standard alias Snek.Board # Two-player game snake_ids = MapSet.new([&quot;snek1&quot;, &quot;snek2&quot;]) # Initialize the game {:ok, turn0} = Standard.init(Board.Size.medium, snake_ids) # The ruleset emits `Snek.Board` states that you can analyze in various ways: Board.empty?(turn0) # =&gt; false Board.alive_snakes_remaining(turn0) # =&gt; 2 # Apply moves for two turns turn1 = Standard.next(turn0, [{&quot;snek1&quot;, :west}, {&quot;snek2&quot;, :north}]) turn2 = Standard.next(turn1, [{&quot;snek1&quot;, :south}, {&quot;snek2&quot;, :east}]) # Is the game over? (See if you can figure out *why* the game isn&#39;t over!) Standard.done?(turn2) # =&gt; falseAll rulesets implement the same callbacks, so you can perform dynamic dispatch. In the following example, the ruleset variable could be set to any ruleset module name:ruleset = Snek.Ruleset.Solo {:ok, turn0} = ruleset.init(Board.Size.medium, snake_ids)","ref":"readme.html#examples","title":"Snek - Examples","type":"extras"},{"doc":"If you have any questions, or just wish to geek out and chat about Battlesnake or Elixir or programming in general, feel free to reach out! I love talking with people and sharing tips and tricks.You can reach me at xtagon@gmail.com, or catch me in Battlesnake Slack (username: @xtagon).","ref":"readme.html#open-invite","title":"Snek - Open Invite","type":"extras"},{"doc":"The following Mix tasks are available to assist in development:mix docsmix testmix coverallsmix credo --strict","ref":"readme.html#development","title":"Snek - Development","type":"extras"},{"doc":"This project is released under the terms of the MIT License.","ref":"readme.html#license","title":"Snek - License","type":"extras"},{"doc":"ChangelogAll notable changes to this project will be documented in this file.The format is based on Keep a Changelog, and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"AddedA Snek.Board.Snake.step/2 function for finding points relative to a snake's last moved direction, but without moving the snake.","ref":"changelog.html#0-2-0-2020-08-30","title":"Changelog - 0.2.0 - 2020-08-30","type":"extras"},{"doc":"AddedA Snek.Board module (and submodules) for representing board positions.A Snek.Ruleset behaviour module for implementing variations of game rules.A Snek.Ruleset.Standard module implementing the Battlesnake Standard rules.A Snek.Ruleset.Solo module implementing the Battlesnake Solo rules.","ref":"changelog.html#0-1-0-2020-08-30","title":"Changelog - 0.1.0 - 2020-08-30","type":"extras"},{"doc":"MIT License Copyright (c) 2020 Justin Workman Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]